let _ = Mltop.add_known_module "coq-elpi.elpi"

# 7 "src/coq_elpi_arg_syntax.mlg"
 

open Ltac_plugin
open Gramlib

open Pcoq.Constr
open Pcoq.Prim
open Pvernac.Vernac_
open Pltac

module EA = Coq_elpi_arg_HOAS
module U = Coq_elpi_utils

(* Arguments ************************************************************* *)
let pr_elpi_string _ _ _ (s : Elpi.API.Ast.Loc.t * string) = Pp.str (snd s)

let trim_loc loc =
  let open Loc in
  { loc with bp = loc.bp + 1; ep = loc.ep - 1 }

let idents_of loc s =
  let s = String.sub s 1 (String.length s - 2) in
  let l = Str.(split (regexp "[\t \n]+") s) in
  List.iter (fun x -> if not (CLexer.is_ident x) then raise Stream.Failure) l;
  Coq_elpi_utils.of_coq_loc (trim_loc loc), l

let rec strip_curly loc s =
  if s.[0] = '\123' then strip_curly (trim_loc loc) String.(sub s 1 (length s - 2))
  else Coq_elpi_utils.of_coq_loc loc, s
let rec strip_round loc s =
  if s.[0] = '(' then strip_round (trim_loc loc) String.(sub s 1 (length s - 2))
  else Coq_elpi_utils.of_coq_loc loc, s
let rec strip_square loc s =
  if s.[0] = '[' then strip_square (trim_loc loc) String.(sub s 1 (length s - 2))
  else Coq_elpi_utils.of_coq_loc loc, s



let (wit_elpi_string, elpi_string) = Tacentries.argument_extend ~plugin:"coq-elpi.elpi" ~name:"elpi_string" 
                                     {
                                     Tacentries.arg_parsing = Vernacextend.Arg_rules (
                                                              []);
                                     Tacentries.arg_tag = None;
                                     Tacentries.arg_intern = Tacentries.ArgInternFun (fun ist v -> (ist, v));
                                     Tacentries.arg_subst = Tacentries.ArgSubstFun (fun s v -> v);
                                     Tacentries.arg_interp = Tacentries.ArgInterpRet;
                                     Tacentries.arg_printer = ((fun env sigma -> 
                                                              
# 45 "src/coq_elpi_arg_syntax.mlg"
                                         pr_elpi_string 
                                                              ), (fun env sigma -> 
                                                              
# 45 "src/coq_elpi_arg_syntax.mlg"
                                         pr_elpi_string 
                                                              ), (fun env sigma -> 
                                                              
# 45 "src/coq_elpi_arg_syntax.mlg"
                                         pr_elpi_string 
                                                              ));
                                     }
let _ = (wit_elpi_string, elpi_string)

let _ = let () =
        Egramml.grammar_extend ~plugin_uid:("coq-elpi.elpi", "coq_elpi_arg_syntax.mlg:0")
        elpi_string
        (Pcoq.Fresh
        (Gramlib.Gramext.First, [(None, None,
                                 [Pcoq.Production.make
                                  (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                  ((Pcoq.Symbol.token (Tok.PSTRING (None)))))
                                  (fun s loc -> 
# 56 "src/coq_elpi_arg_syntax.mlg"
                   
    Coq_elpi_utils.of_coq_loc loc, s
  
                                                );
                                 Pcoq.Production.make
                                 (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                 ((Pcoq.Symbol.token (Tok.PQUOTATION "lp:"))))
                                 (fun s loc -> 
# 49 "src/coq_elpi_arg_syntax.mlg"
                            
    let loc = { loc with Loc.bp = loc.Loc.bp + 3 } in
    if s.[0] = '\123' then strip_curly loc s
    else if s.[0] = '(' then strip_round loc s
    else if s.[0] = '[' then strip_square loc s
    else Coq_elpi_utils.of_coq_loc loc, s
  
                                               )])]))
        in ()


# 62 "src/coq_elpi_arg_syntax.mlg"
 
let pr_fp _ _ _ (_,x) = U.pr_qualified_name x
let pp_elpi_loc _ _ _ (l : Loc.t) = Pp.mt ()

let the_qname = ref ""
let any_qname kwstate strm =
  let re = Str.regexp "[A-Za-z][A-Za-z0-9]*\\(\\.?[A-Za-z][A-Za-z0-9]*\\)*" in
  match LStream.peek kwstate strm with
  | Some (Tok.KEYWORD sym) when Str.string_match re sym 0 ->
      let pos = LStream.count strm in
      let _, ep = Loc.unloc (LStream.get_loc pos strm) in
      LStream.junk kwstate strm;
      begin match LStream.peek kwstate strm with
      | Some (Tok.IDENT id) ->
          let bp, _ = Loc.unloc (LStream.get_loc (pos+1) strm) in
          if Int.equal ep bp then
            (LStream.junk kwstate strm; the_qname := sym ^ id)
          else
            the_qname := sym
      | _ -> the_qname := sym
      end
  | _ -> raise Stream.Failure
let any_qname = Pcoq.Entry.(of_parser "any_qname" { parser_fun = any_qname })



let (wit_qualified_name, qualified_name) = Tacentries.argument_extend ~plugin:"coq-elpi.elpi" ~name:"qualified_name" 
                                           {
                                           Tacentries.arg_parsing = Vernacextend.Arg_rules (
                                                                    []);
                                           Tacentries.arg_tag = None;
                                           Tacentries.arg_intern = Tacentries.ArgInternFun (fun ist v -> (ist, v));
                                           Tacentries.arg_subst = Tacentries.ArgSubstFun (fun s v -> v);
                                           Tacentries.arg_interp = Tacentries.ArgInterpRet;
                                           Tacentries.arg_printer = ((fun env sigma -> 
                                                                    
# 88 "src/coq_elpi_arg_syntax.mlg"
                                            pr_fp 
                                                                    ), (fun env sigma -> 
                                                                    
# 88 "src/coq_elpi_arg_syntax.mlg"
                                            pr_fp 
                                                                    ), (fun env sigma -> 
                                                                    
# 88 "src/coq_elpi_arg_syntax.mlg"
                                            pr_fp 
                                                                    ));
                                           }
let _ = (wit_qualified_name, qualified_name)

let _ = let () =
        Egramml.grammar_extend ~plugin_uid:("coq-elpi.elpi", "coq_elpi_arg_syntax.mlg:1")
        qualified_name
        (Pcoq.Fresh
        (Gramlib.Gramext.First, [(None, None,
                                 [Pcoq.Production.make
                                  (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                  ((Pcoq.Symbol.nterm any_qname)))
                                  (fun _ loc -> 
# 94 "src/coq_elpi_arg_syntax.mlg"
                     loc, Str.split_delim (Str.regexp_string ".") !the_qname 
                                                );
                                 Pcoq.Production.make
                                 (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Tok.PKEYWORD ("_")))))
                                                 ((Pcoq.Symbol.list0 (Pcoq.Symbol.token (Tok.PFIELD (None))))))
                                 (fun s _ loc -> 
# 93 "src/coq_elpi_arg_syntax.mlg"
                                loc, match s with [] -> [] | _ -> "_" :: s 
                                                 );
                                 Pcoq.Production.make
                                 (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Tok.PIDENT (None)))))
                                                 ((Pcoq.Symbol.list0 (Pcoq.Symbol.token (Tok.PFIELD (None))))))
                                 (fun s i loc -> 
# 92 "src/coq_elpi_arg_syntax.mlg"
                                      loc, i :: s 
                                                 )])]))
        in ()

let (wit_elpi_loc, elpi_loc) = Tacentries.argument_extend ~plugin:"coq-elpi.elpi" ~name:"elpi_loc" 
                               {
                               Tacentries.arg_parsing = Vernacextend.Arg_rules (
                                                        [(Pcoq.Production.make
                                                          (Pcoq.Rule.stop)
                                                          (fun loc -> 
# 100 "src/coq_elpi_arg_syntax.mlg"
           loc 
                                                                    ))]);
                               Tacentries.arg_tag = None;
                               Tacentries.arg_intern = Tacentries.ArgInternFun (fun ist v -> (ist, v));
                               Tacentries.arg_subst = Tacentries.ArgSubstFun (fun s v -> v);
                               Tacentries.arg_interp = Tacentries.ArgInterpRet;
                               Tacentries.arg_printer = ((fun env sigma -> 
                                                        
# 99 "src/coq_elpi_arg_syntax.mlg"
             pp_elpi_loc 
                                                        ), (fun env sigma -> 
                                                        
# 99 "src/coq_elpi_arg_syntax.mlg"
             pp_elpi_loc 
                                                        ), (fun env sigma -> 
                                                        
# 99 "src/coq_elpi_arg_syntax.mlg"
             pp_elpi_loc 
                                                        ));
                               }
let _ = (wit_elpi_loc, elpi_loc)


# 103 "src/coq_elpi_arg_syntax.mlg"
 

let telescope = Pcoq.Entry.make "elpi:telescope"
let colon_sort = Pcoq.Entry.make "elpi:colon_sort"
let colon_constr = Pcoq.Entry.make "elpi:colon_constr"

let any_attribute : Attributes.vernac_flags Attributes.attribute =
  Attributes.make_attribute (fun x -> [],x)

let ignore_unknown_attributes extra =
  CWarnings.with_warn "unsupported-attributes" Attributes.unsupported_attributes extra

let skip_attribute : (Str.regexp list option * Str.regexp list option) Attributes.attribute =
  let open Attributes.Notations in
  let o2l = function None -> [] | Some l -> l in
  Attributes.attribute_of_list
    ["skip",
      fun ?loc old -> function
      | Attributes.VernacFlagLeaf (Attributes.FlagString rex) -> Str.regexp rex :: o2l old
      | _ -> CErrors.user_err ?loc (Pp.str "Syntax error, use #[skip=\"rex\"]")]
  ++
  Attributes.attribute_of_list
   ["only",
      fun ?loc old -> function
      | Attributes.VernacFlagLeaf (Attributes.FlagString rex) -> Str.regexp rex :: o2l old
      | _ -> CErrors.user_err ?loc (Pp.str "Syntax error, use #[only=\"rex\"]")]

let synterp_attribute : EA.phase option Attributes.attribute =
  let open EA in
  Attributes.attribute_of_list
    ["phase",
      (fun ?loc old -> function
      | Attributes.VernacFlagLeaf (Attributes.FlagString "parsing") -> Synterp
      | Attributes.VernacFlagLeaf (Attributes.FlagString "execution") -> Interp
      | Attributes.VernacFlagLeaf (Attributes.FlagString "both") -> Both
      | _ -> CErrors.user_err ?loc (Pp.str "Syntax error, use #[phase=\"parsing\"] or #[phase=\"execution\"] or #[phase=\"both\"]"))
    ;"phases",
      (fun ?loc old -> function
      | Attributes.VernacFlagLeaf (Attributes.FlagString "both") -> Both
      | _ -> CErrors.user_err ?loc (Pp.str "Syntax error, use #[phases=\"both\"]"))
    ;"synterp",
      (fun ?loc old -> function
      | Attributes.VernacFlagEmpty -> Synterp
      | _ -> CErrors.user_err ?loc (Pp.str "Syntax error, use #[synterp]"))
    ;"interp",
      (fun ?loc old -> function
      | Attributes.VernacFlagEmpty -> Interp
      | _ -> CErrors.user_err ?loc (Pp.str "Syntax error, use #[interp]"))
    ;"both",
      (fun ?loc old -> function
      | Attributes.VernacFlagEmpty -> Both
      | _ -> CErrors.user_err ?loc (Pp.str "Syntax error, use #[both]"))

    ]

let skip_and_synterp_attributes = Attributes.Notations.(skip_attribute ++ synterp_attribute)

let raw_args_attributes =
  Attributes.(qualify_attribute "arguments" (bool_attribute ~name:"raw"))

let validate_attributes a flags =
  let extra, raw_args = Attributes.parse_with_extra a flags in
  ignore_unknown_attributes extra;
  raw_args

let coq_kwd_or_symbol = Pcoq.Entry.make "elpi:kwd_or_symbol"

let opt2list = function None -> [] | Some l -> l

let the_kwd = ref ""
let any_kwd kwstate strm =
  match LStream.peek kwstate strm with
  | Some (Tok.KEYWORD sym) when sym <> "." -> LStream.junk kwstate strm; the_kwd := sym
  | _ -> raise Stream.Failure
let any_kwd = Pcoq.Entry.(of_parser "any_symbols_or_kwd" { parser_fun = any_kwd })

let pr_attributes _ _ _ atts =
  Pp.(prlist_with_sep (fun () -> str ",") Attributes.pr_vernac_flag atts)

let wit_elpi_ftactic_arg = EA.Tac.wit

let def_body = G_vernac.def_body

let of_coq_inductive_declaration ~atts kind id =
  let open Vernacentries in let open Preprocessed_Mind_decl in
  match preprocess_inductive_decl ~atts kind [id] with
  | Inductive i -> i
  | Record _ -> assert false

let of_coq_record_declaration ~atts kind id =
  let open Vernacentries in let open Preprocessed_Mind_decl in
  match preprocess_inductive_decl ~atts kind [id] with
  | Inductive _ -> assert false
  | Record r -> r

let of_coq_definition ~loc ~atts name udecl def =
  match def with
  | Vernacexpr.DefineBody(bl,red,c,ty) ->
      EA.Cmd.(ConstantDecl { name = snd name; atts; udecl; typ = (bl,ty); red; body = Some c })
  | Vernacexpr.ProveBody _ ->
      CErrors.user_err ~loc Pp.(str"syntax error: missing Definition body")



let _ = let () = assert (Pcoq.Entry.is_empty telescope) in
        let () =
        Egramml.grammar_extend ~plugin_uid:("coq-elpi.elpi", "coq_elpi_arg_syntax.mlg:2")
        telescope
        (Pcoq.Fresh
        (Gramlib.Gramext.First, [(None, None,
                                 [Pcoq.Production.make
                                  (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                  ((Pcoq.Symbol.nterm binders)))
                                  (fun bl loc -> 
# 211 "src/coq_elpi_arg_syntax.mlg"
                          bl 
                                                 )])]))
        in let () = assert (Pcoq.Entry.is_empty colon_sort) in
        let () =
        Egramml.grammar_extend ~plugin_uid:("coq-elpi.elpi", "coq_elpi_arg_syntax.mlg:3")
        colon_sort
        (Pcoq.Fresh
        (Gramlib.Gramext.First, [(None, None,
                                 [Pcoq.Production.make
                                  (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                  ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                                  ((Pcoq.Symbol.nterm sort)))
                                  (fun s _ loc -> 
# 214 "src/coq_elpi_arg_syntax.mlg"
                           s 
                                                  )])]))
        in let () = assert (Pcoq.Entry.is_empty colon_constr) in
        let () =
        Egramml.grammar_extend ~plugin_uid:("coq-elpi.elpi", "coq_elpi_arg_syntax.mlg:4")
        colon_constr
        (Pcoq.Fresh
        (Gramlib.Gramext.First, [(None, None,
                                 [Pcoq.Production.make
                                  (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                  ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                                  ((Pcoq.Symbol.nterm lconstr)))
                                  (fun s _ loc -> 
# 217 "src/coq_elpi_arg_syntax.mlg"
                              s 
                                                  )])]))
        in let () = assert (Pcoq.Entry.is_empty coq_kwd_or_symbol) in
        let () =
        Egramml.grammar_extend ~plugin_uid:("coq-elpi.elpi", "coq_elpi_arg_syntax.mlg:5")
        coq_kwd_or_symbol
        (Pcoq.Fresh
        (Gramlib.Gramext.First, [(None, None,
                                 [Pcoq.Production.make
                                  (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                  ((Pcoq.Symbol.nterm any_kwd)))
                                  (fun _ loc -> 
# 220 "src/coq_elpi_arg_syntax.mlg"
                     !the_kwd 
                                                )])]))
        in ()


# 223 "src/coq_elpi_arg_syntax.mlg"
 

[%%if coq = "8.19" || coq = "8.20"]
let mkAttributesFlagQualid q = Attributes.FlagIdent (Names.Id.to_string (Libnames.qualid_basename q)) 
[%%else]
let mkAttributesFlagQualid q = Attributes.FlagQualid q
[%%endif]



let (wit_attributes, attributes) = Tacentries.argument_extend ~plugin:"coq-elpi.elpi" ~name:"attributes" 
                                   {
                                   Tacentries.arg_parsing = Vernacextend.Arg_rules (
                                                            []);
                                   Tacentries.arg_tag = None;
                                   Tacentries.arg_intern = Tacentries.ArgInternFun (fun ist v -> (ist, v));
                                   Tacentries.arg_subst = Tacentries.ArgSubstFun (fun s v -> v);
                                   Tacentries.arg_interp = Tacentries.ArgInterpRet;
                                   Tacentries.arg_printer = ((fun env sigma -> 
                                                            
# 234 "src/coq_elpi_arg_syntax.mlg"
               pr_attributes 
                                                            ), (fun env sigma -> 
                                                            
# 234 "src/coq_elpi_arg_syntax.mlg"
               pr_attributes 
                                                            ), (fun env sigma -> 
                                                            
# 234 "src/coq_elpi_arg_syntax.mlg"
               pr_attributes 
                                                            ));
                                   }
let _ = (wit_attributes, attributes)

let _ = let my_attribute_list = Pcoq.Entry.make "my_attribute_list"
        and my_attribute = Pcoq.Entry.make "my_attribute"
        and my_attr_value = Pcoq.Entry.make "my_attr_value"
        in
        let () = assert (Pcoq.Entry.is_empty my_attribute_list) in
        let () =
        Egramml.grammar_extend ~plugin_uid:("coq-elpi.elpi", "coq_elpi_arg_syntax.mlg:6")
        my_attribute_list
        (Pcoq.Fresh
        (Gramlib.Gramext.First, [(None, None,
                                 [Pcoq.Production.make
                                  (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                  ((Pcoq.Symbol.list0sep ((Pcoq.Symbol.nterm my_attribute)) ((Pcoq.Symbol.tokens [Pcoq.TPattern (Tok.PKEYWORD (","))])) false)))
                                  (fun a loc -> 
# 238 "src/coq_elpi_arg_syntax.mlg"
                                            a 
                                                )])]))
        in let () = assert (Pcoq.Entry.is_empty my_attribute) in
        let () =
        Egramml.grammar_extend ~plugin_uid:("coq-elpi.elpi", "coq_elpi_arg_syntax.mlg:7")
        my_attribute
        (Pcoq.Fresh
        (Gramlib.Gramext.First, [(None, None,
                                 [Pcoq.Production.make
                                  (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                  ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                                  ("using"))))))
                                                  ((Pcoq.Symbol.nterm my_attr_value)))
                                  (fun v _ loc -> 
# 244 "src/coq_elpi_arg_syntax.mlg"
                                               CAst.make ~loc ("using", v) 
                                                  );
                                 Pcoq.Production.make
                                 (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.nterm ident)))
                                                 ((Pcoq.Symbol.nterm my_attr_value)))
                                 (fun v k loc -> 
# 242 "src/coq_elpi_arg_syntax.mlg"
                                           CAst.make ~loc (Names.Id.to_string k, v) 
                                                 )])]))
        in let () = assert (Pcoq.Entry.is_empty my_attr_value) in
        let () =
        Egramml.grammar_extend ~plugin_uid:("coq-elpi.elpi", "coq_elpi_arg_syntax.mlg:8")
        my_attr_value
        (Pcoq.Fresh
        (Gramlib.Gramext.First, [(None, None,
                                 [Pcoq.Production.make (Pcoq.Rule.stop)
                                  (fun loc -> 
# 251 "src/coq_elpi_arg_syntax.mlg"
             Attributes.VernacFlagEmpty 
                                              );
                                 Pcoq.Production.make
                                 (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                                 ((Pcoq.Symbol.nterm my_attribute_list)))
                                                 ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                                 (fun _ a _ loc -> 
# 250 "src/coq_elpi_arg_syntax.mlg"
                                               Attributes.VernacFlagList a 
                                                   );
                                 Pcoq.Production.make
                                 (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Tok.PKEYWORD ("=")))))
                                                 ((Pcoq.Symbol.nterm qualid)))
                                 (fun v _ loc -> 
# 249 "src/coq_elpi_arg_syntax.mlg"
                              Attributes.VernacFlagLeaf (mkAttributesFlagQualid v) 
                                                 );
                                 Pcoq.Production.make
                                 (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Tok.PKEYWORD ("=")))))
                                                 ((Pcoq.Symbol.nterm string)))
                                 (fun v _ loc -> 
# 248 "src/coq_elpi_arg_syntax.mlg"
                              Attributes.VernacFlagLeaf (Attributes.FlagString v) 
                                                 )])]))
        in let () =
        Egramml.grammar_extend ~plugin_uid:("coq-elpi.elpi", "coq_elpi_arg_syntax.mlg:9")
        attributes
        (Pcoq.Fresh
        (Gramlib.Gramext.First, [(None, None,
                                 [Pcoq.Production.make
                                  (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                  ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm my_attribute)) ((Pcoq.Symbol.tokens [Pcoq.TPattern (Tok.PKEYWORD (","))])) false)))
                                  (fun atts loc -> 
# 255 "src/coq_elpi_arg_syntax.mlg"
                                                               atts 
                                                   )])]))
        in ()

let (wit_elpi_cmd_arg, elpi_cmd_arg) = Tacentries.argument_extend ~plugin:"coq-elpi.elpi" ~name:"elpi_cmd_arg" 
                                       {
                                       Tacentries.arg_parsing = Vernacextend.Arg_rules (
                                                                [(Pcoq.Production.make
                                                                  (Pcoq.Rule.next 
                                                                  (Pcoq.Rule.stop)
                                                                  ((Pcoq.Symbol.nterm coq_kwd_or_symbol)))
                                                                  (fun x
                                                                  loc -> 
                                                                  
# 301 "src/coq_elpi_arg_syntax.mlg"
                                EA.Cmd.String x 
                                                                  ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "("))))
                                                                 ((Pcoq.Symbol.nterm lconstr)))
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal ")"))))
                                                                 (fun _ t _
                                                                 loc -> 
                                                                 
# 299 "src/coq_elpi_arg_syntax.mlg"
                              EA.Cmd.Term t 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "Context"))))
                                                                 ((Pcoq.Symbol.nterm telescope)))
                                                                 (fun ty _
                                                                 loc -> 
                                                                 
# 297 "src/coq_elpi_arg_syntax.mlg"
                                   EA.Cmd.Context ty 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "#["))))
                                                                 ((Pcoq.Symbol.nterm attributes)))
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "]"))))
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "Axiom"))))
                                                                 ((Pcoq.Symbol.nterm qualified_name)))
                                                                 ((Pcoq.Symbol.opt (Pcoq.Symbol.nterm univ_decl))))
                                                                 ((Pcoq.Symbol.nterm telescope)))
                                                                 ((Pcoq.Symbol.nterm colon_constr)))
                                                                 (fun t typ
                                                                 udecl name _
                                                                 _ atts _
                                                                 loc -> 
                                                                 
# 293 "src/coq_elpi_arg_syntax.mlg"
                                                                                                                     
      EA.Cmd.(ConstantDecl { name = snd name; atts; udecl; typ = (typ,Some t); red = None; body = None }) 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "Axiom"))))
                                                                 ((Pcoq.Symbol.nterm qualified_name)))
                                                                 ((Pcoq.Symbol.opt (Pcoq.Symbol.nterm univ_decl))))
                                                                 ((Pcoq.Symbol.nterm telescope)))
                                                                 ((Pcoq.Symbol.nterm colon_constr)))
                                                                 (fun t typ
                                                                 udecl name _
                                                                 loc -> 
                                                                 
# 291 "src/coq_elpi_arg_syntax.mlg"
                                                                                           
      EA.Cmd.(ConstantDecl { name = snd name; atts = []; udecl; typ = (typ,Some t); red = None; body = None }) 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "#["))))
                                                                 ((Pcoq.Symbol.nterm attributes)))
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "]"))))
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "Definition"))))
                                                                 ((Pcoq.Symbol.nterm qualified_name)))
                                                                 ((Pcoq.Symbol.opt (Pcoq.Symbol.nterm univ_decl))))
                                                                 ((Pcoq.Symbol.nterm def_body)))
                                                                 (fun def
                                                                 udecl name _
                                                                 _ atts _
                                                                 loc -> 
                                                                 
# 289 "src/coq_elpi_arg_syntax.mlg"
                                                                                                          of_coq_definition ~loc ~atts name udecl def 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "Definition"))))
                                                                 ((Pcoq.Symbol.nterm qualified_name)))
                                                                 ((Pcoq.Symbol.opt (Pcoq.Symbol.nterm univ_decl))))
                                                                 ((Pcoq.Symbol.nterm def_body)))
                                                                 (fun def
                                                                 udecl name _
                                                                 loc -> 
                                                                 
# 288 "src/coq_elpi_arg_syntax.mlg"
                                                                                of_coq_definition ~loc ~atts:[] name udecl def 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "#["))))
                                                                 ((Pcoq.Symbol.nterm attributes)))
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "]"))))
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "Structure"))))
                                                                 ((Pcoq.Symbol.nterm inductive_or_record_definition)))
                                                                 (fun id _ _
                                                                 atts _
                                                                 loc -> 
                                                                 
# 286 "src/coq_elpi_arg_syntax.mlg"
                                                                                    EA.Cmd.RecordDecl (of_coq_record_declaration ~atts Vernacexpr.Structure id) 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "Structure"))))
                                                                 ((Pcoq.Symbol.nterm inductive_or_record_definition)))
                                                                 (fun id _
                                                                 loc -> 
                                                                 
# 285 "src/coq_elpi_arg_syntax.mlg"
                                                          EA.Cmd.RecordDecl (of_coq_record_declaration ~atts:[] Vernacexpr.Structure id) 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "#["))))
                                                                 ((Pcoq.Symbol.nterm attributes)))
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "]"))))
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "Class"))))
                                                                 ((Pcoq.Symbol.nterm inductive_or_record_definition)))
                                                                 (fun id _ _
                                                                 atts _
                                                                 loc -> 
                                                                 
# 283 "src/coq_elpi_arg_syntax.mlg"
                                                                                EA.Cmd.RecordDecl (of_coq_record_declaration ~atts Vernacexpr.(Class true) id) 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "Class"))))
                                                                 ((Pcoq.Symbol.nterm inductive_or_record_definition)))
                                                                 (fun id _
                                                                 loc -> 
                                                                 
# 282 "src/coq_elpi_arg_syntax.mlg"
                                                      EA.Cmd.RecordDecl (of_coq_record_declaration ~atts:[] Vernacexpr.(Class true) id) 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "#["))))
                                                                 ((Pcoq.Symbol.nterm attributes)))
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "]"))))
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "Record"))))
                                                                 ((Pcoq.Symbol.nterm inductive_or_record_definition)))
                                                                 (fun id _ _
                                                                 atts _
                                                                 loc -> 
                                                                 
# 280 "src/coq_elpi_arg_syntax.mlg"
                                                                                 EA.Cmd.RecordDecl (of_coq_record_declaration ~atts Vernacexpr.Record id) 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "Record"))))
                                                                 ((Pcoq.Symbol.nterm inductive_or_record_definition)))
                                                                 (fun id _
                                                                 loc -> 
                                                                 
# 279 "src/coq_elpi_arg_syntax.mlg"
                                                       EA.Cmd.RecordDecl (of_coq_record_declaration ~atts:[] Vernacexpr.Record id) 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "#["))))
                                                                 ((Pcoq.Symbol.nterm attributes)))
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "]"))))
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "Variant"))))
                                                                 ((Pcoq.Symbol.nterm inductive_or_record_definition)))
                                                                 (fun id _ _
                                                                 atts _
                                                                 loc -> 
                                                                 
# 277 "src/coq_elpi_arg_syntax.mlg"
                                                                                  EA.Cmd.IndtDecl (of_coq_inductive_declaration ~atts Vernacexpr.Variant id) 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "Variant"))))
                                                                 ((Pcoq.Symbol.nterm inductive_or_record_definition)))
                                                                 (fun id _
                                                                 loc -> 
                                                                 
# 276 "src/coq_elpi_arg_syntax.mlg"
                                                        EA.Cmd.IndtDecl (of_coq_inductive_declaration ~atts:[] Vernacexpr.Variant id) 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "#["))))
                                                                 ((Pcoq.Symbol.nterm attributes)))
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "]"))))
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "CoInductive"))))
                                                                 ((Pcoq.Symbol.nterm inductive_or_record_definition)))
                                                                 (fun id _ _
                                                                 atts _
                                                                 loc -> 
                                                                 
# 274 "src/coq_elpi_arg_syntax.mlg"
                                                                                      EA.Cmd.IndtDecl (of_coq_inductive_declaration ~atts Vernacexpr.CoInductive id) 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "CoInductive"))))
                                                                 ((Pcoq.Symbol.nterm inductive_or_record_definition)))
                                                                 (fun id _
                                                                 loc -> 
                                                                 
# 273 "src/coq_elpi_arg_syntax.mlg"
                                                            EA.Cmd.IndtDecl (of_coq_inductive_declaration ~atts:[] Vernacexpr.CoInductive id) 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "#["))))
                                                                 ((Pcoq.Symbol.nterm attributes)))
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "]"))))
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "Inductive"))))
                                                                 ((Pcoq.Symbol.nterm inductive_or_record_definition)))
                                                                 (fun id _ _
                                                                 atts _
                                                                 loc -> 
                                                                 
# 271 "src/coq_elpi_arg_syntax.mlg"
                                                                                    EA.Cmd.IndtDecl (of_coq_inductive_declaration ~atts Vernacexpr.Inductive_kw id) 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Pcoq.terminal "Inductive"))))
                                                                 ((Pcoq.Symbol.nterm inductive_or_record_definition)))
                                                                 (fun id _
                                                                 loc -> 
                                                                 
# 270 "src/coq_elpi_arg_syntax.mlg"
                                                          EA.Cmd.IndtDecl (of_coq_inductive_declaration ~atts:[] Vernacexpr.Inductive_kw id) 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.nterm string)))
                                                                 (fun s
                                                                 loc -> 
                                                                 
# 268 "src/coq_elpi_arg_syntax.mlg"
                     EA.Cmd.String s 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.nterm integer)))
                                                                 (fun n
                                                                 loc -> 
                                                                 
# 267 "src/coq_elpi_arg_syntax.mlg"
                      EA.Cmd.Int n 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.nterm qualified_name)))
                                                                 (fun s
                                                                 loc -> 
                                                                 
# 266 "src/coq_elpi_arg_syntax.mlg"
                             EA.Cmd.String (String.concat "." (snd s)) 
                                                                 ))]);
                                       Tacentries.arg_tag = None;
                                       Tacentries.arg_intern = Tacentries.ArgInternFun ((fun f ist v -> (ist, f ist v)) (
                                                               
# 262 "src/coq_elpi_arg_syntax.mlg"
                EA.Cmd.glob 
                                                               ));
                                       Tacentries.arg_subst = Tacentries.ArgSubstFun (
                                                              
# 263 "src/coq_elpi_arg_syntax.mlg"
                 EA.Cmd.subst 
                                                              );
                                       Tacentries.arg_interp = Tacentries.ArgInterpSimple (
                                                               
# 261 "src/coq_elpi_arg_syntax.mlg"
                 EA.Cmd.interp 
                                                               );
                                       Tacentries.arg_printer = ((fun env sigma -> 
                                                                
# 264 "src/coq_elpi_arg_syntax.mlg"
                 fun _ _ _ -> EA.Cmd.pp_raw env sigma 
                                                                ), (fun env sigma -> 
                                                                
# 265 "src/coq_elpi_arg_syntax.mlg"
                  fun _ _ _ -> EA.Cmd.pp_glob env sigma 
                                                                ), (fun env sigma -> 
                                                                
# 260 "src/coq_elpi_arg_syntax.mlg"
             fun _ _ _ -> EA.Cmd.pp_top env sigma 
                                                                ));
                                       }
let _ = (wit_elpi_cmd_arg, elpi_cmd_arg)

let (wit_elpi_tactic_arg, elpi_tactic_arg) = Tacentries.argument_extend ~plugin:"coq-elpi.elpi" ~name:"elpi_tactic_arg" 
                                             {
                                             Tacentries.arg_parsing = 
                                             Vernacextend.Arg_rules (
                                             [(Pcoq.Production.make
                                               (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Pcoq.terminal "ltac_tactic"))))
                                                               ((Pcoq.Symbol.token (Pcoq.terminal ":"))))
                                                               ((Pcoq.Symbol.token (Pcoq.terminal "("))))
                                                               ((Pcoq.Symbol.nterm ltac_expr)))
                                                               ((Pcoq.Symbol.token (Pcoq.terminal ")"))))
                                               (fun _ t _ _ _ loc -> 
# 316 "src/coq_elpi_arg_syntax.mlg"
                                                  EA.Tac.LTacTactic t 
                                                                    ));
                                             (Pcoq.Production.make
                                              (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Pcoq.terminal "ltac_term_list"))))
                                                              ((Pcoq.Symbol.token (Pcoq.terminal ":"))))
                                                              ((Pcoq.Symbol.token (Pcoq.terminal "("))))
                                                              ((Pcoq.Symbol.nterm ident)))
                                                              ((Pcoq.Symbol.token (Pcoq.terminal ")"))))
                                              (fun _ t _ _ _ loc -> 
# 315 "src/coq_elpi_arg_syntax.mlg"
                                                 EA.Tac.LTac(EA.Tac.List EA.Tac.Term,(CAst.make ~loc @@ Constrexpr.CRef (Libnames.qualid_of_string ~loc @@ Names.Id.to_string t,None))) 
                                                                    ));
                                             (Pcoq.Production.make
                                              (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Pcoq.terminal "ltac_term"))))
                                                              ((Pcoq.Symbol.token (Pcoq.terminal ":"))))
                                                              ((Pcoq.Symbol.token (Pcoq.terminal "("))))
                                                              ((Pcoq.Symbol.nterm ident)))
                                                              ((Pcoq.Symbol.token (Pcoq.terminal ")"))))
                                              (fun _ t _ _ _ loc -> 
# 314 "src/coq_elpi_arg_syntax.mlg"
                                            EA.Tac.LTac(EA.Tac.Term, CAst.make ~loc @@ Constrexpr.CRef (Libnames.qualid_of_string ~loc @@ Names.Id.to_string t,None)) 
                                                                    ));
                                             (Pcoq.Production.make
                                              (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Pcoq.terminal "ltac_int_list"))))
                                                              ((Pcoq.Symbol.token (Pcoq.terminal ":"))))
                                                              ((Pcoq.Symbol.token (Pcoq.terminal "("))))
                                                              ((Pcoq.Symbol.nterm ident)))
                                                              ((Pcoq.Symbol.token (Pcoq.terminal ")"))))
                                              (fun _ t _ _ _ loc -> 
# 313 "src/coq_elpi_arg_syntax.mlg"
                                                EA.Tac.LTac(EA.Tac.List EA.Tac.Int, (CAst.make ~loc @@ Constrexpr.CRef (Libnames.qualid_of_string ~loc @@ Names.Id.to_string t,None))) 
                                                                    ));
                                             (Pcoq.Production.make
                                              (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Pcoq.terminal "ltac_int"))))
                                                              ((Pcoq.Symbol.token (Pcoq.terminal ":"))))
                                                              ((Pcoq.Symbol.token (Pcoq.terminal "("))))
                                                              ((Pcoq.Symbol.nterm ident)))
                                                              ((Pcoq.Symbol.token (Pcoq.terminal ")"))))
                                              (fun _ t _ _ _ loc -> 
# 312 "src/coq_elpi_arg_syntax.mlg"
                                           EA.Tac.LTac(EA.Tac.Int, (CAst.make ~loc @@ Constrexpr.CRef (Libnames.qualid_of_string ~loc @@ Names.Id.to_string t,None))) 
                                                                    ));
                                             (Pcoq.Production.make
                                              (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Pcoq.terminal "ltac_string_list"))))
                                                              ((Pcoq.Symbol.token (Pcoq.terminal ":"))))
                                                              ((Pcoq.Symbol.token (Pcoq.terminal "("))))
                                                              ((Pcoq.Symbol.nterm ident)))
                                                              ((Pcoq.Symbol.token (Pcoq.terminal ")"))))
                                              (fun _ t _ _ _ loc -> 
# 311 "src/coq_elpi_arg_syntax.mlg"
                                                   EA.Tac.LTac(EA.Tac.List EA.Tac.String, (CAst.make ~loc @@ Constrexpr.CRef (Libnames.qualid_of_string ~loc @@ Names.Id.to_string t,None))) 
                                                                    ));
                                             (Pcoq.Production.make
                                              (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Pcoq.terminal "ltac_string"))))
                                                              ((Pcoq.Symbol.token (Pcoq.terminal ":"))))
                                                              ((Pcoq.Symbol.token (Pcoq.terminal "("))))
                                                              ((Pcoq.Symbol.nterm ident)))
                                                              ((Pcoq.Symbol.token (Pcoq.terminal ")"))))
                                              (fun _ t _ _ _ loc -> 
# 310 "src/coq_elpi_arg_syntax.mlg"
                                              EA.Tac.LTac(EA.Tac.String, (CAst.make ~loc @@ Constrexpr.CRef (Libnames.qualid_of_string ~loc @@ Names.Id.to_string t,None))) 
                                                                    ));
                                             (Pcoq.Production.make
                                              (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Pcoq.terminal "("))))
                                                              ((Pcoq.Symbol.nterm lconstr)))
                                                              ((Pcoq.Symbol.token (Pcoq.terminal ")"))))
                                              (fun _ t _ loc -> 
# 309 "src/coq_elpi_arg_syntax.mlg"
                              EA.Tac.Term t 
                                                                ));
                                             (Pcoq.Production.make
                                              (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.nterm string)))
                                              (fun s loc -> 
# 308 "src/coq_elpi_arg_syntax.mlg"
                     EA.Tac.String s 
                                                            ));
                                             (Pcoq.Production.make
                                              (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.nterm integer)))
                                              (fun n loc -> 
# 307 "src/coq_elpi_arg_syntax.mlg"
                      EA.Tac.Int n 
                                                            ));
                                             (Pcoq.Production.make
                                              (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.nterm qualified_name)))
                                              (fun s loc -> 
# 306 "src/coq_elpi_arg_syntax.mlg"
                             EA.Tac.String (String.concat "." (snd s)) 
                                                            ))]);
                                             Tacentries.arg_tag = Some
                                                                  (Geninterp.val_tag (Genarg.topwit wit_elpi_ftactic_arg));
                                             Tacentries.arg_intern = 
                                             Tacentries.ArgInternWit (wit_elpi_ftactic_arg);
                                             Tacentries.arg_subst = Tacentries.ArgSubstWit (wit_elpi_ftactic_arg);
                                             Tacentries.arg_interp = 
                                             Tacentries.ArgInterpWit (wit_elpi_ftactic_arg);
                                             Tacentries.arg_printer = 
                                             ((fun env sigma -> fun _ _ _ _ -> Pp.str "missing printer"), (fun env sigma -> fun _ _ _ _ -> Pp.str "missing printer"), (fun env sigma -> fun _ _ _ _ -> Pp.str "missing printer"));
                                             }
let _ = (wit_elpi_tactic_arg, elpi_tactic_arg)

let (wit_ltac_attributes, ltac_attributes) = Tacentries.argument_extend ~plugin:"coq-elpi.elpi" ~name:"ltac_attributes" 
                                             {
                                             Tacentries.arg_parsing = 
                                             Vernacextend.Arg_rules (
                                             [(Pcoq.Production.make
                                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm ident)))
                                               (fun v loc -> 
# 329 "src/coq_elpi_arg_syntax.mlg"
                    (CAst.make ~loc @@ Constrexpr.CRef (Libnames.qualid_of_string ~loc @@ Names.Id.to_string v,None)) 
                                                             ))]);
                                             Tacentries.arg_tag = None;
                                             Tacentries.arg_intern = 
                                             Tacentries.ArgInternFun ((fun f ist v -> (ist, f ist v)) (
                                             
# 325 "src/coq_elpi_arg_syntax.mlg"
                  fun gsig t -> fst @@ Ltac_plugin.Tacintern.intern_constr gsig t 
                                             ));
                                             Tacentries.arg_subst = Tacentries.ArgSubstFun (
                                                                    
# 326 "src/coq_elpi_arg_syntax.mlg"
                   fun x -> Detyping.subst_glob_constr (Global.env()) x 
                                                                    );
                                             Tacentries.arg_interp = 
                                             Tacentries.ArgInterpSimple (
# 321 "src/coq_elpi_arg_syntax.mlg"
                   fun ist env evd x -> match DAst.get x with
      | Glob_term.GVar id ->
          Ltac_plugin.Tacinterp.interp_ltac_var (Ltac_plugin.Tacinterp.Value.cast (Genarg.topwit wit_attributes)) ist None (CAst.make id)
      | _ -> assert false 
                                             );
                                             Tacentries.arg_printer = 
                                             ((fun env sigma -> 
# 327 "src/coq_elpi_arg_syntax.mlg"
                   fun _ _ _ -> Ppconstr.pr_constr_expr env sigma 
                                             ), (fun env sigma -> 
# 328 "src/coq_elpi_arg_syntax.mlg"
                    fun _ _ _ -> Printer.pr_glob_constr_env env sigma 
                                             ), (fun env sigma -> 
# 320 "src/coq_elpi_arg_syntax.mlg"
               pr_attributes 
                                             ));
                                             }
let _ = (wit_ltac_attributes, ltac_attributes)

